@@ -35,7 +35,7 @@
 #include "transform.h"
 
 /* gettimeofday on Windows ??? */
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
 #ifdef _MSC_VER
 #include <winsock2.h>
 #pragma comment(lib, "ws2_32.lib")
@@ -291,6 +291,8 @@ xsltPrintErrorContext(xsltTransformContextPtr ctxt,
     const xmlChar *name = NULL;
     const char *type = "error";
 
+    if (ctxt != NULL)
+	ctxt->state = XSLT_STATE_ERROR;
     if ((node == NULL) && (ctxt != NULL))
 	node = ctxt->inst;
 
@@ -922,7 +924,7 @@ xsltSaveResultTo(xmlOutputBufferPtr buf, xmlDocPtr result,
  * @compression:  the compression factor (0 - 9 included)
  *
  * Save the result @result obtained by applying the @style stylesheet
- * to a file or URL @URL
+ * to a file or @URL
  *
  * Returns the number of byte written or -1 in case of failure.
  */
